from boto3.session import Session


_aws_session = Session(
    region_name='eu-central-1',
    aws_access_key_id='AKIAXCSJV3BS6BWMC54H',
    aws_secret_access_key='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'
)


class S3():

    @staticmethod
    def get_secure_url(bucket, filename, expires_in=7200, insecure=False):
        global _aws_session

        # max is 7 days
        if not expires_in:
            expires_in = 7200
        elif expires_in > 604800:
            expires_in = 604800

        url = _aws_session.client('s3').generate_presigned_url(
            'get_object',
            Params={
                'Bucket': bucket, 'Key': filename},
            ExpiresIn=expires_in
        )

        if insecure:
            return url.split("?")[0]
        else:
            return url



from .message_log import Log
from .config import load_config
from os import path, system
import subprocess
import yaml


class SDKSetup:
    config = load_config('config.yml')
    home_path = path.expanduser('~')
    config_path = path.expanduser('~/.grepsr/config.yml')

    def __init__(self, type, dryrun, sdk_image):
        self.type_dict = self.config[type]
        self.type_name = type
        self.dryrun = dryrun
        self.sdk_image = sdk_image

        self.do_setup()

    def check_path(self):
        for service_dir in self.type_dict['paths']:
            service_dir = path.expanduser(service_dir)
            if not path.exists(service_dir):
                Log.error(
                    f'Directory {service_dir} was not found')
                Log.error(
                    "Please check ~/.grepsr/config.yml to check if paths are correctly added")
                return False
        return True

    def generate_autocomplete_directory(self, docker_id, dir_name):
        for service_dir in self.type_dict['paths']:
            Log.info(
                f"Adding autocomplete modules in {service_dir}")
            command = f'''
                    rm -rf {service_dir}/.autocomplete && 
                    mkdir {service_dir}/.autocomplete
                '''

            command += f"""docker cp -L {docker_id}:/home/grepsr/{dir_name}/lib {service_dir}/.autocomplete/
            docker cp -L {docker_id}:/home/grepsr/{dir_name}/vendor {service_dir}/.autocomplete/
            """
            system(command)

    def do_setup(self):
        if self.sdk_image is not None:
            with open(self.config_path, 'w') as w:
                self.config[self.type_name]['sdk_image'] = self.sdk_image
                w.write(yaml.dump(self.config))

        if(self.check_path() == False):
            return False
        if(self.dryrun == False):
            Log.info("Logging in to AWS service")

            system(f'''export AWS_ACCESS_KEY_ID={self.config['aws_access_key_id']} &&
            export AWS_SECRET_ACCESS_KEY={self.config['aws_secret_access_key']}
            eval $(aws ecr get-login --no-include-email --region eu-central-1)''')

            Log.info("Getting the SDK")
            system(f"docker pull {self.type_dict['sdk_image']}")

        docker_id = subprocess.run(
            ['docker',  'create', self.type_dict['sdk_image']], stdout=subprocess.PIPE).stdout
        docker_id = docker_id.decode('utf-8').rstrip()

        if self.type_name == 'php':
            self.generate_autocomplete_directory(
                docker_id, 'vortex-backend')
        elif self.type_name == 'php_next':
            self.generate_autocomplete_directory(
                docker_id, 'vortex-backend-next')

        system(f'docker rm {docker_id}')
        Log.info("SDK has been installed successfully")



from threading import Condition
from grepsrcli.core.config import load_config
from webbrowser import open_new_tab
from flask import Flask, request, render_template


def load_ui(port):
    """this is main function for UI portion

     Args:
         port (int): the port in which the UI needs to be launched in the system
    """
    app = Flask(__name__)

    @app.route("/")
    def main():

        try:
            config = load_config('config.yml')
            platforms = {
                'PHP': config['php'],
                'Node': config['node'],
                'Python': config['python'],
                'PHP_Next': config['php_next']
            }

            return render_template('form2.html', config=config, platforms=platforms)
        except:
            return render_template('form.html')

    @app.route("/api")
    def api():
        return " Hello World API"

    @app.route('/config', methods=['POST'])
    def config():
        if request.method == 'POST':
            # process config and save it to configuration file

            f = request.environ.get("werkzeug.server.shutdown")
            f()
            return render_template('config.html')

    open_new_tab('0.0.0.0:' + str(port))
    app.run(debug=False, host="0.0.0.0", port=port)



from cement import App, TestApp, init_defaults
from cement.core.exc import CaughtSignal
from .core.exc import GrepsrCliError
from .controllers.base import Base
from .controllers.crawler import Crawler, CrawlerBase
from .controllers.report import Report, ReportBase

# configuration defaults
CONFIG = init_defaults('grepsrcli')

class GrepsrCli(App):
    """Grepsr Cli primary application."""

    class Meta:
        label = 'gcli'

        # configuration defaults
        config_defaults = CONFIG

        # call sys.exit() on close
        exit_on_close = True

        # load additional framework extensions
        extensions = [
            'yaml',
            'colorlog',
            'jinja2',
        ]

        # configuration handler
        config_handler = 'yaml'

        # configuration file suffix
        config_file_suffix = '.yml'

        # set the log handler
        log_handler = 'colorlog'

        # set the output handler
        output_handler = 'jinja2'

        # register handlers
        handlers = [
            Base,
            CrawlerBase,
            Crawler,
            ReportBase,
            Report
        ]


class GrepsrCliTest(TestApp, GrepsrCli):
    """A sub-class of GrepsrCli that is better suited for testing."""

    class Meta:
        label = 'grepsrcli'


def main():
    with GrepsrCli() as app:
        try:
            app.run()

        except AssertionError as e:
            print('AssertionError > %s' % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback
                traceback.print_exc()

        except GrepsrCliError as e:
            print('GrepsrCliError > %s' % e.args[0])
            app.exit_code = 1

            if app.debug is True:
                import traceback
                traceback.print_exc()

        except CaughtSignal as e:
            # Default Cement signals are SIGINT and SIGTERM, exit 0 (non-error)
            print('\n%s' % e)
            app.exit_code = 0


if __name__ == '__main__':
    main()



from cement import Controller, ex
from webbrowser import open_new_tab
from ..core.config import load_config
from ..core.utils import get_plugin_path, get_plugin_info
from ..core.report_api import get_report_from_pid, run_report_by_rid


class ReportBase(Controller):
    class Meta:
        label = 'report_base'


class Report(Controller):
    config = load_config('config.yml')

    class Meta:
        label = 'report'
        stacked_on = 'report_base'
        stacked_type = 'nested'

    @ex(
        help="run report to live",
        arguments=[
            (['-rid', '--report-id'], {
                "action": "store",
                "dest": "report_id"
            }),
            (['-s', '--service'], {
                "action": "store",
                "dest": "service_code"
            })
        ]
    )
    def run(self):

        if(self.app.pargs.service_code is not None):
            service_code = self.app.pargs.service_code
            plugin_info = get_plugin_info(
                get_plugin_path(service_code, all_types=True))

            rid = get_report_from_pid(
                plugin_info['pid'], plugin_info['report_name'])

            if(rid):
                run_api_message = run_report_by_rid(rid)
                self.app.log.info("Report: {}, message: {}".format(
                    service_code, run_api_message))
                open_new_tab(
                    f'https://appnext.grepsr.com/projects/{plugin_info["pid"]}')
            else:
                self.app.log.warning(
                    "We couldn't run report via report name, please try running it manually via the url")

                self.app.log.info(
                    f'App Url: https://appnext.grepsr.com/projects/{plugin_info["pid"]}')

        elif (self.app.pargs.report_id is not None):
            report_id = self.app.pargs.report_id
            run_api_message = run_report_by_rid(report_id)
            self.app.log.info("Report ID: {}, message: {}".format(
                report_id, run_api_message))
        else:
            self.app.log.error("please use a valid syntax")



from grepsrcli.core.message_log import Log
from cement import Controller, ex
import pkg_resources
from grepsrcli.core.sdk_setup import SDKSetup
from grepsrcli.core.config import save_config
from grepsrcli.core.UI.app import load_ui


version = pkg_resources.require("grepsr-cli")[0].version

VERSION_BANNER = """
gcli: cli tool for grepsr developers verion: %s
""" % (version)


class Base(Controller):

    class Meta:
        label = 'base'

        arguments = [
            (['-v', '--version'],
             {'action': 'version',
                'version': VERSION_BANNER}),
        ]

    def _default(self):
        self.app.args.print_help()

    @ex(help="setup SDKs for crawling",
        arguments=[
            (['-t', '--type'], {'action': 'store', 'dest': 'type'}),
            (['--dryrun'], {'action': 'store_true',  'dest': 'dryrun'}),
            (['--sdk'], {'action': 'store',  'dest': 'sdk'}),
        ]
        )
    def setup_sdk(self):

        if self.app.pargs.type is not None:
            SDKSetup(self.app.pargs.type,
                     self.app.pargs.dryrun, self.app.pargs.sdk)
        else:
            Log.error(
                "Please select the platform to setup the sdk.\nExample: gcli setup-sdk -t php|php_next")

    @ex(help="Update configuration for gcli",
        arguments=[
            (['--update'], {'action': 'store_true',  'dest': 'update'}),
            (['--gui'], {'action': 'store_true',  'dest': 'gui'}),
            (['--port'], {'action': 'store',  'dest': 'port'})
        ]
        )
    def configure(self):
        if self.app.pargs.gui == False:
            if(self.app.pargs.update == True):
                save_config(update=True)
            else:
                save_config(update=False)
        else:
            port = self.app.pargs.port
            if not port:
                port = 5000
            load_ui(port)
    


from os import makedirs, system, path, stat
import subprocess
import time
import pathlib
from semver import VersionInfo, process
from cement import Controller, ex
from ..core.config import load_config
from ..core.utils import create_boilerplate, render_boilerplate, get_plugin_info, get_plugin_path, show_schema
from ..core.test_local import TestLocal
from ..core.message_log import Log
from ..core.aws_s3 import S3


class CrawlerBase(Controller):
    class Meta:
        label = 'crawler_base'


class Crawler(Controller):
    config = load_config('config.yml')

    class Meta:
        label = 'crawler'
        stacked_on = 'crawler_base'
        stacked_type = 'nested'

    @ex(
        help="test a plugin locally",
        arguments=[
            (['-s'], {
                'action': 'store',
                'dest': 'plugin_name',
                'help': "The name of the service/plugin to run locally"
            }),
        ]
    )
    def test(self):

        plugin_name = self.app.pargs.plugin_name

        if get_plugin_path(plugin_name, type='php'):
            base_path = pathlib.Path(get_plugin_path(
                plugin_name, type='php')).parent
            TestLocal('php', plugin_name, base_path)
        elif get_plugin_path(plugin_name, type='php_next'):

            base_path = pathlib.Path(get_plugin_path(
                plugin_name, type='php_next')).parent
            TestLocal('php_next', plugin_name, base_path)
        else:
            self.app.log.error("path: {} not found!".format(plugin_name))

    @ex(
        help="performs git pull to update the codebase",
        arguments=[
            (['-t', '--type'], {
                'action': 'store',
                'dest': 'type',
                'help': "Platform type: php|node|python|php_next"
            })
        ]
    )
    def sync(self):
        if(self.app.pargs.type):
            type = self.app.pargs.type
            crawler_paths = self.config[type]['paths']

            for crawler_path in crawler_paths:
                Log.info(f"Syncing: {crawler_path}")
                system(
                    f"""cd {crawler_path}  && git pull origin master""")
        else:
            Log.warn("Please enter a valid type")

    @ex(
        help="create basic boilerplate for plugins ",
        arguments=[
            (['-s'], {
                "action": "store",
                "dest": "plugin_name",
                "help": "the name of the plugin to be created"
            }), (['-pid'], {
                "action": "store",
                "dest": 'pid',
                "help": "the project id of the plugin"
            }), (
                ['-t', '--template'], {
                    "action": "store",
                    "dest": 'template',
                    "help": "choose a template to boilerplate php|node|py|vc defaults to php"
                }
            ), (
                ['--path'], {
                    'action': "store",
                    'dest': 'folder_path',
                    'help': 'the path of the folder where the plugin will reside'
                }
            )
        ]
    )
    def create(self):

        template = self.app.pargs.template

        if self.app.pargs.folder_path is not None:
            folder_path = self.app.pargs.folder_path
            folder_path = folder_path.rstrip('/')
            folder_path = path.expanduser(folder_path)

        else:
            self.app.log.error(
                "cannot create boilerplate file without destination path")
            return

        if self.app.pargs.plugin_name is not None:
            plugin_name = self.app.pargs.plugin_name
        else:
            self.app.log.error(
                "cannot create boilerplate file without plugin's name")
            return

        if self.app.pargs.pid is not None:
            pid = self.app.pargs.pid
        else:
            pid = '***'

        data = {
            'plugin_name': plugin_name,
            'pid': pid
        }
        plugin_path = folder_path + '/' + plugin_name
        if template == 'vc':
            create_boilerplate(
                plugin_path, "php_vc_boilerplate.jinja2", data, 'php')

        elif template == 'py':
            create_boilerplate(
                plugin_path, "py_boilerplate.jinja2", data, 'py')

        elif template == 'node':
            create_boilerplate(
                plugin_path, "node_boilerplate.jinja2", data, 'js')
        else:
            create_boilerplate(
                plugin_path, "php_boilerplate.jinja2", data, 'php')

    @ex(
        help="deploy a specific plugin to live with versioning",
        arguments=[
            (['-s'], {
                "action": "store",
                "dest": 'plugin_name'
            }), (['-m'], {
                "action": "store",
                "dest": "message"
            }), (['-st', '--stable'], {
                "action": "store_true",
                "dest": "stable_flag"
            }), (['--patch'], {
                "action": "store_true",
                "dest": "patch_flag"
            }), (['--minor'], {
                "action": "store_true",
                "dest": "minor_flag",
                "help": "set major of a version"
            }), (['--major'], {
                "action": "store_true",
                "dest": "major_flag"
            }),
            (['--verbose'], {
                "action": 'store_true',
                "dest": "verbose"
            })
        ]
    )
    def deploy(self):
        input_plugin_name = self.app.pargs.plugin_name
        deploy_message = self.app.pargs.message

        if input_plugin_name is None:
            self.app.log.error(
                "cannot deploy without service code")
            return False
        if deploy_message is None:
            self.app.log.error(
                "cannot deploy without deploy message")
            return False

        if self.app.pargs.stable_flag:
            deploy_type = "DEPLOY-STABLE"
        else:
            deploy_type = "DEPLOY"

        major_flag = self.app.pargs.major_flag
        minor_flag = self.app.pargs.minor_flag

        plugin_names = False
        if(',' in input_plugin_name):
            plugin_names = input_plugin_name.split(',')

        if(plugin_names == False):
            plugin_names = []
            plugin_names.append(input_plugin_name)

        for plugin_name in plugin_names:
            print(f'Deploying: {plugin_name} ')

            plugin_name = plugin_name.strip()

            plugin_dir_path = get_plugin_path(plugin_name, all_types=True)

            if(plugin_dir_path):

                plugin_info = get_plugin_info(plugin_dir_path)
                version_path = '{}/.version'.format(plugin_dir_path)
                if(path.exists(version_path)):
                    with open(version_path, 'r') as f:
                        version_info = VersionInfo.parse(f.read())
                        if(major_flag):
                            version_info = version_info.next_version(
                                part='major')
                        elif(minor_flag):
                            version_info = version_info.next_version(
                                part='minor')
                        else:
                            version_info = version_info.bump_patch()

                        version_info = str(version_info)
                        with open(version_path, 'w') as w:
                            w.write(version_info)

                    self.app.log.info(
                        "[{}] [{}] {}".format(deploy_type, version_info, deploy_message))
                else:
                    with open(version_path, 'w') as f:
                        if(major_flag):
                            version_info = "1.0.0"
                        elif (minor_flag):
                            version_info = "0.1.0"
                        else:
                            version_info = "0.0.1"

                        f.write(version_info)

                    self.app.log.info(
                        "[{}] [{}] {}".format(deploy_type, version_info, deploy_message))
                try:
                    cmd = f"""
                        cd {plugin_dir_path} &&
                        cd .. &&
                        git stash --quiet &&
                        git pull origin master --quiet &&
                        git stash apply --quiet &&
                        git add {plugin_name}/  &&
                        git commit -m "[{deploy_type}] [{version_info}] {deploy_message}" --quiet &&
                        git push origin master --quiet
                        """

                    if self.app.pargs.verbose is True:
                        cmd = f"""
                        cd {plugin_dir_path} &&
                        cd .. &&
                        git stash &&
                        git pull origin master &&
                        git stash apply &&
                        git add {plugin_name}/ &&
                        git commit -m "[{deploy_type}] [{version_info}] {deploy_message}" &&
                        git push origin master
                        """

                    subprocess.run(
                        cmd, shell=True, check=True)

                except subprocess.CalledProcessError:
                    self.app.log.error(
                        f'There was a problem deploying {plugin_name}')
                    return

                show_schema(plugin_dir_path)
                # sleep for a while to fake a real deployment.
                try:
                    app_url = f'https://appnext.grepsr.com/projects/{plugin_info["pid"]}'
                    self.app.log.info(f"App Url: {app_url}")
                    self.app.log.info(
                        f"Plugin: {plugin_name} deployed successfully")
                except:
                    self.app.log.warning(
                        f"Cannot find pid in plugin, please find the project's url manually")
                    self.app.log.info(
                        f"Plugin: {plugin_name} deployed successfully")

    @ex(
        help="use a base plugin from: https://bitbucket.org/grepsr/vortex-plugins-services-base",
        arguments=[
            (['-s'], {
                "help": "name of the base crawler you want to use",
                "action": "store",
                "dest": "base_plugin_name"
            }),
            (['-t'], {
                "help": "name of target crawler that you want to use",
                "action": "store",
                "dest": "plugin_name"
            })
        ]
    )
    def use_basecrawler(self):

        s3 = S3()

        version = None
        base_plugin_name = self.app.pargs.base_plugin_name
        target_plugin_name = self.app.pargs.plugin_name

        if(base_plugin_name is None or target_plugin_name is None):
            Log.error(
                "Please Enter base crawler and target cralwers.\nUse gcli crawler use-basecrawler -h for more information")
            return

        if('-' in base_plugin_name):
            version = base_plugin_name.split('-')[1]
            base_plugin_name = base_plugin_name.split('-')[0]

        # to check if there is a base-plugin in thier gcli settings ...
        base_plugin_dir_path = get_plugin_path(
            base_plugin_name, type='php')

        target_plugin_path = get_plugin_path(
            target_plugin_name, type='php'
        )

        if(version is None and base_plugin_dir_path):
            f = open(f"{base_plugin_dir_path}/.version", mode='r')
            version = f.read()
            f.close()

        if(version is None):
            Log.error(
                "Please Specify the version of base crawler that you want to use")
            return

        if(target_plugin_path):
            Log.info(
                f'Getting Secure Url of target plugin: {base_plugin_name}, {version}')

            presigned_url = s3.get_secure_url(bucket='crawler-plugins',
                                              filename=f'php/vortex-plugins-services-base/{base_plugin_name}/{base_plugin_name}-{version}.tar.gz')
            data = {
                'presigned_url': presigned_url,
                'plugin_name': base_plugin_name,
                'version': version
            }

            render_boilerplate(boilerplate='composer.jinja2', data=data,
                               destination_path=target_plugin_path + '/composer.json')
            system(
                f""" cd {target_plugin_path} &&  composer install""")
        else:
            Log.error(f"Target plugin: {target_plugin_name} not found")



import yaml
from os import path, remove
from pathlib import Path
from .message_log import Log
import prompt
from jinja2 import Template


config_path = path.expanduser('~/.grepsr')


def load_config(file_name, file_path=None):
    """Loads yml config file for user configuration

    Args:
        file_name (str): the name of file in the path gcli/config
    """

    if file_path is None:
        file_path = config_path
    try:
        file = f'{file_path}/{file_name}'
        with open(file, 'r') as f:
            file_content = f.read()
            return yaml.full_load(file_content)
    except:
        Log.error("{}  not found in Path {}".format(file_name, file_path))


def save_config(update=False):

    if update == False:
        setup_config = {}
    else:
        setup_config = load_config('config.yml')

    platforms = ['php', 'node', 'python', 'php_next']
    crawler_paths = []

    for plat in platforms:
        if _yes_no_prompt(f"\nProcess configuration for {Log.OKCYAN + plat + Log.ENDC} platform?"):
            if update == False:
                setup_config[plat] = {}
                setup_config[plat]['paths'] = []

                if(plat == 'php'):
                    setup_config[plat]['sdk_image'] = '486559570021.dkr.ecr.eu-central-1.amazonaws.com/vortex/sdk:stable'
                elif(plat == 'php_next'):
                    setup_config[plat]['sdk_image'] = '486559570021.dkr.ecr.eu-central-1.amazonaws.com/vortex/backend-next:latest'
                else:
                    setup_config[plat]['sdk_image'] = ''

            setup_config[plat]['env'] = {}
            Log.heading("\nStep 1: Path Configuration")

            if(update == True):
                if(_yes_no_prompt("Do you want to update crawler paths?") == True):
                    setup_config[plat]['paths'] = []
                    while True:
                        base_path = prompt.string(
                            prompt="Enter path for your plugin directory: ")
                        base_path = path.expanduser(base_path.rstrip('/'))
                        setup_config[plat]['paths'].append(base_path)
                        crawler_paths.append(base_path)
                        if(_yes_no_prompt("Do you want to another path?") == False):
                            break
            elif(update == False):
                while True:
                    base_path = prompt.string(
                        prompt="Enter path for your plugin directory: ")
                    base_path = path.expanduser(base_path.rstrip('/'))
                    setup_config[plat]['paths'].append(base_path)
                    crawler_paths.append(base_path)
                    if(_yes_no_prompt("Do you want to another path?") == False):
                        break
            else:
                pass

            Log.heading("\nStep 2: SDK Configuration")
            if(update == True):
                if(_yes_no_prompt("Do you want to update SDK image ?") == True):
                    if(plat == 'php'):
                        setup_config[plat]['sdk_image'] = '486559570021.dkr.ecr.eu-central-1.amazonaws.com/vortex/sdk:stable'
                    elif(plat == 'php_next'):
                        setup_config[plat]['sdk_image'] = '486559570021.dkr.ecr.eu-central-1.amazonaws.com/vortex/backend-next:latest'
                    else:
                        setup_config[plat]['sdk_image'] = ''

                    sdk_image = prompt.string(
                        prompt=f"Enter SDK image for {Log.OKCYAN + plat + Log.ENDC} (defaults to stable sdk if empty): ", empty=True)
                    if(sdk_image is not None):
                        setup_config[plat]['sdk_image'] = sdk_image
            elif(update == False):
                sdk_image = prompt.string(
                    prompt=f"Enter SDK image for {Log.OKCYAN + plat + Log.ENDC} (defaults to stable sdk if empty): ", empty=True)
                if(sdk_image is not None):
                    setup_config[plat]['sdk_image'] = sdk_image
            else:
                pass

            Log.heading("\nStep 3: Environment variable Coniguration")
            if(_yes_no_prompt("Do you want to add environment variables?") == True):
                while True:
                    env_key_val = prompt.string(
                        f"Enter env variable for {Log.OKCYAN + plat + Log.ENDC} (Eg:- CFG_PARAMS_SELENIUM_HOST: 192.168.90.64) \n")
                    try:
                        env_key_val = env_key_val.split(':')
                        setup_config[plat]['env'][env_key_val[0].strip()
                                                  ] = env_key_val[1].strip()
                    except:
                        Log.error("Please follow the correct format")
                    if(_yes_no_prompt("Do you want to another env variable?") == False):
                        break

    if(update == True):
        if(_yes_no_prompt("Do you want to update your aws keys?") == True):
            setup_config['aws_access_key_id'] = prompt.string(
                f"Enter aws id \n")
            setup_config['aws_secret_access_key'] = prompt.string(
                f"Enter aws secret key \n")
    else:
        setup_config['aws_access_key_id'] = prompt.string(
            f"Enter aws id \n")
        setup_config['aws_secret_access_key'] = prompt.string(
            f"Enter aws secret key \n")

    setup_config['app_env'] = 'tst'

    with open(config_path + '/config.yml', 'w') as w:
        w.write(yaml.dump(setup_config))

    if(update == False):
        _generate_autocomplete_script(crawler_paths)


def _yes_no_prompt(ques):
    answer = prompt.character(prompt=ques + ' (y/n) ', empty=True)
    if(answer == 'y' or answer is None):
        return True
    else:
        return False


def _generate_autocomplete_script(crawler_paths):
    Log.info("Generating Autocomplete Script")
    data = {}
    data['paths'] = []
    for p in range(len(crawler_paths)):
        d = {}
        d['ind'] = p
        d['dest_path'] = crawler_paths[p]
        data['paths'].append(d)

    if path.exists(config_path + '/autocomplete.sh'):
        remove(config_path +
               '/autocomplete.sh')

    template_dir = Path(__file__).parent.parent.absolute()
    template_file = '{}/templates/{}'.format(
        template_dir, 'autocomplete.jinja2')

    with open(template_file) as file:
        template = Template(file.read())

    with open(config_path + '/autocomplete.sh', 'w') as dest_file:
        dest_file.write(template.render(data))



from os import path, system
from .message_log import Log
from .config import load_config


class TestLocal:
    def __init__(self, type, plugin_name, base_path):
        self.type = type
        self.plugin_name = plugin_name
        self.base_path = base_path

        self.tmp_path = path.expanduser('~') + '/.grepsr/tmp/'
        self.config = load_config('config.yml')
        self.type_config = self.config[type]

        self.test_script()

    def test_script(self):
        if self.type == "php":
            if "env" in self.type_config:
                if self.type_config['env']:
                    self.test_script_php(env=True)
                else:
                    self.test_script_php(env=False)
            else:
                self.test_script_php(env=False)
        elif self.type == "php_next":
            self.test_script_php_next()

    def test_script_php(self, env=False):

        if env == True:
            env_vars = [
                f'-e {env_var}={self.type_config["env"][env_var]}' for env_var in self.type_config["env"].keys()]
            command = f'''docker run -t -i --network="host" --rm -v {self.base_path}:/home/grepsr/vortex-plugins/{path.basename(self.base_path)} -v {self.tmp_path}:/tmp -e APP_ENV={self.config['app_env']}  {" ".join(env_vars)} {self.type_config['sdk_image']} -s {self.plugin_name}'''

        else:
            command = f'''docker run -t -i --network="host" --rm  \
                    -v {self.base_path}:/home/grepsr/vortex-plugins/{path.basename(self.base_path)}  \
                    -v {self.tmp_path}:/tmp  \
                    -e APP_ENV={self.config['app_env']}  \
                    {self.type_config['sdk_image']} -s {self.plugin_name}'''

        system(command)

    def test_script_php_next(self):

        command = f'''docker run -t -i --network="host" --rm \
                     -v {self.base_path}:/home/grepsr/vortex-backend-next/scraper-plugins \
                     -v {self.tmp_path}:/tmp \
                     -e APP_ENV={self.config['app_env']} \
                     {self.type_config['sdk_image']} -s {self.plugin_name} '''

        system(command)



from os import path, mkdir
from setuptools import setup, find_packages


with open('.version', 'r') as f:
    VERSION = f.read()


f = open('README.md', 'r')
LONG_DESCRIPTION = f.read()
f.close()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

config_folder = path.expanduser('~') + '/.grepsr'
if not path.exists(config_folder):
    mkdir(config_folder)

if not path.exists(config_folder + '/tmp'):
    mkdir(config_folder + '/tmp')

print("\033[95mThankyou for installing gcli\033[0m\n")

setup(
    name='grepsr-cli',
    install_requires=requirements,
    version=VERSION,
    description='A Cli tool for Grepsr Developers',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    author='grepsr',
    author_email='dev@grepsr.com',
    url='https://bitbucket.org/grepsr/grepsr-cli/',
    license='unlicensed',
    packages=find_packages(exclude=['ez_setup', 'tests*']),
    package_data={'grepsrcli': ['templates/*']},
    include_package_data=True,
    entry_points="""
        [console_scripts]
        gcli = grepsrcli.main:main
    """
)



class Log:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

    @staticmethod
    def info(message):
        print(f'{Log.OKGREEN}INFO: {message}{Log.ENDC}')

    @staticmethod
    def error(message):
        print(f'{Log.BOLD}{Log.FAIL}ERR: {message}{Log.ENDC}')

    @staticmethod
    def standout(message):
        print(f'{Log.HEADER}{message}{Log.ENDC}')

    @staticmethod
    def success(message):
        print(f'{Log.OKGREEN}SUCCESS: {message}{Log.ENDC}')

    @staticmethod
    def warn(message):
        print(f'{Log.WARNING}WARNING: {message}{Log.ENDC}')

    @staticmethod
    def heading(message):
        print(f'{Log.OKBLUE}{message}{Log.ENDC}')



import json
from pathlib import Path
from os import listdir, path
import re
from jinja2 import Template
from os import path, mkdir
from .config import load_config
from terminaltables import SingleTable
from .message_log import Log


def get_plugin_path(plugin_name, type='php', all_types=False):
    """ returns the path of the plugin if plugin name exist

    Args:
        plugin_name (str): name of the service plugin .
        type (str, optional): php|py|node. Defaults to 'php'.
        all_types: retuns the path regardless the plugin types i.e php, py and node
    """
    config = load_config('config.yml')
    if all_types == False:
        if type == 'php':
            if _generate_plugin_path(plugin_name, config['php']['paths']):
                return _generate_plugin_path(plugin_name, config['php']['paths'])
            else:
                return False
        elif type == 'node':
            if _generate_plugin_path(plugin_name, config['node']['paths']):
                return _generate_plugin_path(plugin_name, config['node']['paths'])
            else:
                return False
        elif type == 'py':
            if _generate_plugin_path(plugin_name, config['python']['paths']):
                return _generate_plugin_path(plugin_name, config['python']['paths'])
            else:
                return False
        elif type == 'php_next':
            if _generate_plugin_path(plugin_name, config['php_next']['paths']):
                return _generate_plugin_path(plugin_name, config['php_next']['paths'])
            else:
                return False
        else:
            Log.error("Invalid params for type.")
            return False
    else:
        # todo: do not hardcode this
        base_paths_list = [
            config['php']['paths'] if 'php' in config else None,
            config['node']['paths'] if 'node' in config else None,
            config['python']['paths'] if 'python' in config else None,
            config['php_next']['paths'] if 'php_next' in config else None
        ]

        for base_paths in base_paths_list:
            if base_paths is not None and _generate_plugin_path(plugin_name, base_paths):
                return _generate_plugin_path(plugin_name, base_paths)
        Log.error("Plugin name not found.")
        return False


def create_boilerplate(folder_path, boilerplate, data, extention,):
    """ to create boilerplate for a given path

    Args:
        folder_path (str): the destination path
        boilerplate (str): the name of the boilerplate
        data (dict): the data to be rendered
    """

    service_name = path.basename(folder_path)
    try:
        mkdir(folder_path)
    except OSError as err:
        Log.error(err)
        return

    dest_path = '{}/{}.{}'.format(
                folder_path, service_name, extention)

    render_boilerplate(boilerplate=boilerplate, data=data,
                       destination_path=dest_path)

    Log.info(
        'Plugin created at {}'.format(dest_path))


def show_schema(base_path):
    """ this method will show the schema of a plugin if it has schema.json

    Args:
        base_path (str): the base path of the plugin  eg: /home/vtx-services/aaa_com/
    """

    schema_path = base_path + '/schema.json'
    if path.exists(schema_path):
        try:
            with open(schema_path, 'r') as f:
                schema = f.read()
                schema = json.loads(schema)
                for page in schema.keys():
                    Log.standout(f"Schema for Page: {page}")
                    schema_heading = ['field', 'type', 'pattern']
                    table_data = [
                        schema_heading
                    ]

                    for th, td in schema[page]['schema']['properties'].items():
                        if 'pattern' in td:
                            row = [th, td['type'], td['pattern']]
                        else:
                            row = [th, td['type'], '']

                        table_data.append(row)

                    print(SingleTable(table_data).table)
                return True
        except:
            Log.warn("Schema Structured Incorrectly")
    else:
        Log.warn("Schema Not Found")
        return False


def get_plugin_info(plugin_path):
    """ get plugin's info like service_name,pid,description from the plugin's base folder.
    It does so by reading the file and looking at the info from plugin which is commented
    at the beginning.

    Args:
        plugin_path (str): the path of the directory where we find the plugin.
    """
    plugin_file_path = listdir(plugin_path)

    files = [f for f in plugin_file_path]
    plugin_file = ''
    for file in files:
        if path.basename(plugin_path) in file:
            plugin_file = file
            break

    if plugin_file:
        try:
            with open(f'{plugin_path}/{plugin_file}') as f:
                script = f.read()

                pid = re.search(r'pid\s*\:\s*([0-9]+)', script, re.IGNORECASE)
                pid = pid[1]

                report_name = re.search(
                    r'name\s*\:\s*(.*)', script, re.IGNORECASE)
                report_name = report_name[1]

                desc = re.search(r'description\s*\:\s*(.*)',
                                 script, re.IGNORECASE)
                desc = desc[1]

                return {
                    'pid': pid,
                    'report_name': report_name,
                    'desc': desc
                }
        except:
            pass


def render_boilerplate(boilerplate, data, destination_path):
    """parse boilerplate from template directory to start a project

    Args:
        boilerplate (str): name of boilerplate template
        data (dict): input for the boilerplate
        destination_path: the final path where the final content needs to be saved
    """

    template_dir = Path(__file__).parent.parent.absolute()
    template_file = '{}/templates/{}'.format(template_dir, boilerplate)
    with open(template_file) as file:
        template = Template(file.read())
        with open(destination_path, 'w') as dest_file:
            dest_file.write(template.render(data))


def _generate_plugin_path(plugin_name, paths):
    for service_path in paths:
        plugin_path = f'{service_path}/{plugin_name}'
        plugin_path = path.expanduser(plugin_path)
        if(path.exists(plugin_path)):
            return plugin_path
    return False



from ..core.config import load_config
import requests as req
import json


config = load_config('config.yml')


def get_report_from_pid(pid, report_name):
    """[summary]

    Args:
        pid ([type]): [description]
        report_id ([type]): [description]
    """

    res = req.get(
        'https://api.grepsr.com/v1/report/list?project_id={}&x-api-key={}'.format(pid, config['api_key']))

    res_json = json.loads(res.text)
    reports = res_json['payload']
    for report in reports:
        if report['name'].strip() == report_name.strip():
            return report['report_id']

    return False


def run_report_by_rid(rid):
    """[summary]

    Args:
        rid ([type]): [description]
    """

    run_api = req.get(
        'https://api.grep.sr/v1/report/run?report_id={}&x-api-key={}'.format(rid, config['api_key']))

    return json.loads(run_api.text)['message']
